import os, ConfigParser

@plugin
class Auth(object):
    def __init__(self, server):
        self.server = server
        self.commands = [ "addadmin", "addmod", "deladmin", "delmod", "admins", "mods", "owners", "passwd", "identify" ]
        self.owners = {}
        self.authed_owner_hosts = []
        self.admins = {}
        self.authed_admin_hosts = []
        self.mods = {}
        self.authed_mod_hosts = []
        self.server.handle("command", self.handle_command, self.commands)
        self.server.handle("part", self.handle_part)
        self.server.handle("quit", self.handle_quit)
        self.server.handle("nick", self.handle_nick)
        self.authdir = self.server.config["confdir"] + "network-auth-users/"
        self.authfile = self.authdir + self.server.config["network"] + "-auth.cfg"
        self._loadusers()
    
    def _checkAuth(self, user):
        user = user[0]
        stillOn = False
        for ch in self.server.userlist.keys():
            if user in self.server.userlist[ch]:
                stillOn = True
        
        if not stillOn:
            if user.lower() in self.owners.keys():
                self.owners[user.lower()][1] = False
            elif user.lower() in self.admins.keys():
                self.admins[user.lower()][1] = False
            elif user.lower() in self.mods.keys():
                self.mods[user.lower()][1] = False
    
    def handle_quit(self, user, message):
        self._checkAuth(user)
    
    def handle_part(self, channel, user, message):
        self._checkAuth(user)
    
    def handle_nick(self, oldnick, newnick):
        if self.isOwner(oldnick):
            print self.owners
            self.owners[newnick.lower()] = self.owners[oldnick.lower()]
            del self.owners[oldnick.lower()]
        elif self.isAdmin(oldnick):
            self.admins[newnick.lower()] = self.admins[oldnick.lower()]
            del self.admins[oldnick.lower()]
        elif self.isMod(oldnick):
            self.mods[newnick.lower()] = self.mods[oldnick.lower()]
            del self.mods[oldnick.lower()]
        
    def handle_command(self, channel, user, cmd, args):
        if cmd == "owners":
            self.server.doMessage(channel, user+": My owners are: "+" ".join(self.owners))
        elif cmd == "admins":
            self.server.doMessage(channel, user+": My administrators are: "+" ".join(self.admins))
        elif cmd == "mods":
            self.server.doMessage(channel, user+": My moderators are: "+" ".join(self.mods))
        elif cmd == "passwd":
            if channel == user:
                if user.lower() in self.owners.keys():
                    if self.owners[user.lower()][1]:
                        if len(args) < 1:
                            self.server.doMessage(channel, user+": Not enough arguments.")
                            return
                        
                        if len(args) >= 2:
                            self.owners[user.lower()][0] = args[1]
                            self.server.doMessage(channel, user+": Password successfully changed.")
                        else:
                            self.owners[user.lower()][0] = args[0]
                            self.server.doMessage(channel, user+": Password successfully changed.")
                            
                elif user.lower() in self.admins.keys():
                    if self.admins[user.lower()][1]:
                        if len(args) < 1:
                            self.server.doMessage(channel, user+": Not enough arguments.")
                            return
                        
                        if len(args) >= 2:
                            self.admins[user.lower()][0] = args[1]
                            self.server.doMessage(channel, user+": Password successfully changed.")
                        else:
                            self.admins[user.lower()][0] = args[0]
                            self.server.doMessage(channel, user+": Password successfully changed.")
                            
                elif user.lower() in self.mods.keys():
                    if self.mods[user.lower()][1]:
                        if len(args) < 1:
                            self.server.doMessage(channel, user+": Not enough arguments.")
                            return
                        
                        if len(args) >= 2:
                            self.mods[user.lower()][0] = args[1]
                            self.server.doMessage(channel, user+": Password successfully changed.")
                        else:
                            self.mods[user.lower()][0] = args[0]
                            self.server.doMessage(channel, user+": Password successfully changed.")
                            
                else:
                    self.server.doMessage(channel, user+": You are not in the owners/admins/mods lists.")
                    return
            else:
                self.server.doMessage(channel, user+": Please do that using private messaging.")
                return
        elif cmd == "identify":
            if len(args) < 1:
                self.server.doMessage(channel, user+": Not enough arguments.")
                return
            if channel == user:
                if user.lower() in self.owners.keys():
                    if self.owners[user.lower()][1]:
                        self.server.doMessage(channel, user+": You are already identified.")
                    elif args[0] == self.owners[user.lower()][0]:
                        self.owners[user.lower()][1] = True
                        self.server.doMessage(channel, user+": You are now identified for "+user)
                    else:
                        self.server.doMessage(channel, user+": Invalid password.")
                elif user.lower() in self.admins.keys():
                    if self.admins[user.lower()][1]:
                        self.server.doMessage(channel, user+": You are already identified.")
                    elif args[0] == self.admins[user.lower()][0]:
                        self.admins[user.lower()][1] = True
                        self.server.doMessage(channel, user+": You are now identified for "+user)
                    else:
                        self.server.doMessage(channel, user+": Invalid password.")
                elif user.lower() in self.mods.keys():
                    if self.mods[user.lower()][1]:
                        self.server.doMessage(channel, user+": You are already identified.")
                    elif args[0] == self.mods[user.lower()][0]:
                        self.mods[user.lower()][1] = True
                        self.server.doMessage(channel, user+": You are now identified for "+user)
                    else:
                        self.server.doMessage(channel, user+": Invalid password.")
            else:
                self.server.doMessage(channel, user+": Please do that using private messaging.")
                return
            
        elif self.isOwner(user):
            if len(args) < 1:
                self.server.doMessage(channel, user+": Not enough arguments.")
                return
            
            if cmd == "addadmin":
                if args[0] in self.admins.keys(): return
                self.admins[args[0]] = ['', False]
            elif cmd == "addmod":
                if args[0] in self.mods.keys(): return
                self.mods[args[0]] = ['', False]
            elif cmd == "deladmin":
                if args[0] in self.admins.keys():
                    del self.admins[args[0]]
            elif cmd == "delmod":
                if args[0] in self.mods.keys():
                    del self.mods[args[0]]
    
    #save users
    def __del__(self):
        if not os.path.isdir(self.authdir):
            os.mkdir(self.authdir)
            
        fh = open(self.authfile, "w")
        if not self.configParser.has_section("owners"):
            self.configParser.add_section("owners")
        if not self.configParser.has_section("admins"):
            self.configParser.add_section("admins")
        if not self.configParser.has_section("mods"):
            self.configParser.add_section("mods")
        
        for user in self.owners.keys():
            self.configParser.set("owners",user,self.owners[user][0])
        for user in self.admins.keys():
            self.configParser.set("admins",user,self.admins[user][0])
        for user in self.mods.keys():
            self.configParser.set("mods",user,self.mods[user][0])
        
        self.configParser.write(fh)
        fh.close()
        
    
    def _loadusers(self):
        if os.path.isfile(self.authfile):
            self.configParser = ConfigParser.RawConfigParser()
            self.configParser.read(self.authfile)
                
            for section in self.configParser.sections():
                if section == "owners":
                    for user,pswd in self.configParser.items(section):
                        self.owners[user.lower()] = [pswd, pswd == '']
                elif section == "admins":
                    for user,pswd in self.configParser.items(section):
                        self.admins[user.lower()] = [pswd, pswd == '']
                elif section == "mods":
                    for user,pswd in self.configParser.items(section):
                        self.mods[user.lower()] = [pswd, pswd == '']
            
        onick = self.server.config["owner_nick"].lower()
        if not onick in self.owners.keys():
            if getattr(self, "configParser", None) is None:
                self.configParser = ConfigParser.RawConfigParser()
                self.configParser.add_section("owners")
                self.configParser.add_section("admins")
                self.configParser.add_section("mods")
            
            self.configParser.set("owners", onick, '')
            self.owners[onick] = ['', True]
    
    def isOwner(self, user):
        return user.lower() in self.owners and self.owners[user.lower()][1]
    
    def isAdmin(self, user):
        if user.lower() in self.admins and self.admins[user.lower()][1]: 
            return True
        else:
            return self.isOwner(user)
    
    def isMod(self, user):
        if user.lower() in self.mods and self.mods[user.lower()][1]: 
            return True
        elif self.isAdmin(user): 
            return True
        else:
            return self.isOwner(user)
            
